__version__ = '0.8.3'

from .pearson import pearson
from .kendall import kendall
from .spearman import spearman
from .find_unrelated import find_unrelated
from .flatten import flatten
from .corrgram import corrgram
from .corr_vs_pval import corr_vs_pval
from .bracket_pval import bracket_pval
from .confusion import confusion
from .mcc import mcc, confusion_to_mcc
from .slice_yx import slice_yx
from .find_best import find_best
from .find_worst import find_worst
from .mincorr import mincorr, mincorr_result_check
from .bootcorr import bootcorr
